@@ -22,6 +22,7 @@
 
 class GList;
 class UnicodeMap;
+class UnicodeRemapping;
 
 class TextBlock;
 class TextChar;
@@ -390,6 +391,9 @@ public:
 		      double *xMin, double *yMin,
 		      double *xMax, double *yMax);
 
+  // Returns true if x,y falls inside a column.
+  GBool checkPointInside(double x, double y);
+
   // Find a point inside a column.  Returns false if x,y fall outside
   // all columns.
   GBool findPointInside(double x, double y, TextPosition *pos);
@@ -425,6 +429,11 @@ public:
   // Build a flat word list, in the specified ordering.
   TextWordList *makeWordList();
 
+  // Build a word list containing only words inside the specified
+  // rectangle.
+  TextWordList *makeWordListForRect(double xMin, double yMin,
+				    double xMax, double yMax);
+
   // Returns true if the primary character direction is left-to-right,
   // false if it is right-to-left.
   GBool primaryDirectionIsLR();
@@ -532,8 +541,15 @@ private:
   void dumpUnderlines();
 #endif
 
+  // word list
+  TextWordList *makeWordListForChars(GList *charList);
+
   TextOutputControl control;	// formatting parameters
 
+  UnicodeRemapping *remapping;
+  Unicode *uBuf;
+  int uBufSize;
+
   double pageWidth, pageHeight;	// width and height of current page
   int charPos;			// next character position (within content
 				//   stream)
@@ -686,6 +702,11 @@ public:
   // order (if both flags are false).
   TextWordList *makeWordList();
 
+  // Build a word list containing only words inside the specified
+  // rectangle.
+  TextWordList *makeWordListForRect(double xMin, double yMin,
+				    double xMax, double yMax);
+
   // Returns the TextPage object for the last rasterized page,
   // transferring ownership to the caller.
   TextPage *takeText();
