@@ -34,7 +34,7 @@
 #define SEEK_CUR 1
 #define SEEK_END 2
 
-#define BANNER "\nThis is dvidvi 1.1, Copyright (C) 1988-91, Radical Eye Software\n" \
+#define BANNER "\nThis is dvidvi 1.1, Copyright 1988-1991 Radical Eye Software\n" \
                  "Improved from 1.0 to 1.1, 1994, Markus Kohm\n"
 #define STRINGSIZE (500L)  /* maximum number of strings in program */
 
@@ -73,6 +73,14 @@ void error(const char *);
 #endif
 #include <kpathsea/c-pathch.h>
 #define PATHSEP         ENV_SEP
+#ifdef WIN32
+#include <kpathsea/config.h>
+#include <kpathsea/variable.h>
+#undef fopen
+#undef fprintf
+#define fopen    fsyscp_fopen
+#define fprintf  win32_fprintf
+#endif
 #else  /* not KPATHSEA */
 #if defined(__TOS__)
 #define READBIN         "rb"    /* TOS must use binary mode */
@@ -130,7 +138,7 @@ const char *oname=NULL ;      /* output dvi file name */
 char *iname ;                 /* input dvi file name */
 char *strings ;               /* pointer of the string pool */
 
-char banner[] = BANNER ;      /* the startup message */
+const char banner[] = BANNER ;      /* the startup message */
 integer inlength ;      /* the length of the input dvi file */
 integer postloc ;       /* location of the postamble */
 integer mag ;           /* magnification factor */
@@ -165,6 +173,20 @@ integer *pageloc ;
 integer *pagenumbers ;
 int prettycolumn ;       /* the column we are at when running pretty */
 
+#ifdef ASCIIPTEX
+int ptexdvi ;            /* true if dvi file is extended (TATEKUMI) */
+#endif
+
+#ifdef WIN32
+void win32_fprintf(FILE *fp, const char *fmt, ...) {
+  va_list argp;
+
+  va_start(argp, fmt);
+  win32_vfprintf(fp, fmt, argp);
+  va_end(argp);
+}
+#endif
+
 /*
  *   This array holds values that indicate the length of a command, if
  *   we aren't concerned with that command (which is most of them) or
@@ -750,7 +772,8 @@ case '?' :
             usage() ;
             break ;
 default:
-	    error("! Bad option, not one of filmnqrx?") ;
+	    fprintf(stderr, "dvidvi: Bad option %c\n", c);
+            error("! Try -? for more information") ;
          }  /* switch c */
       } else {  /* this a file name */
          if (*iname == 0) { /* input file name */
@@ -873,6 +896,11 @@ static void readdvifile(void) {
       if (c == 2 && d == 0xdf /* dave fuchs */ &&
                             e == 0xdf)
          break ;
+#ifdef	ASCIIPTEX
+      if (c == 3 && d == 0xdf /* dave fuchs */ &&
+                            e == 0xdf)
+         break ;
+#endif
       fseek(infile,-4L,SEEK_CUR);
       }
    if (p < 10)
@@ -1024,6 +1052,11 @@ static void writepostamble(void) {
          putfontdef(i) ;
    outdvibyte(249) ;
    outdviquad(p) ;
+#ifdef ASCIIPTEX
+   if (ptexdvi)
+      outdvibyte(3) ;
+   else
+#endif
    outdvibyte(2) ;
    outdviquad(0xdfdfdfdfL) ;
    while (dviloc & 3)
@@ -1129,6 +1162,12 @@ case 243: case 244: case 245: case 246:
                p += dvibyte() + 2 ;
                fseek(infile,p,SEEK_SET);
                break ;
+#ifdef ASCIIPTEX
+case 255:
+               ptexdvi = 1 ;
+               outdvibyte(len);
+               break ;
+#endif
 default:       fprintf(stderr, "Bad dvi command was %d at %ld\n", len, p) ;
                error("! lost sync dvi in file lost dvi sync file in") ;
             }
@@ -1160,6 +1199,10 @@ static void writedvifile(void) {
    struct pagespec *ps ;
    Boolean beginp ;
 
+#ifdef ASCIIPTEX
+   ptexdvi = 0 ;
+#endif
+
    writepreamble() ;
    if ( !pagemodulo )
       pagefake = (pagecount + modulo - 1) / modulo * modulo ;
@@ -1217,6 +1260,16 @@ static void writedvifile(void) {
 }
 int main(int argc, char *argv[])
 {
+#if defined(WIN32) && defined(KPATHSEA)
+   int ac;
+   char **av, *enc;
+   kpse_set_program_name(argv[0], "dvidvi");
+   enc = kpse_var_value("command_line_encoding");
+   if (get_command_line_args_utf8(enc, &ac, &av)) {
+      argc = ac;
+      argv = av;
+   }
+#endif
    processargs(argc, argv) ;
    readdvifile() ;
    writedvifile() ;
