@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2008-2018 by Jin-Hwan Cho, Matthias Franz, and Shunsaku Hirata,
+    Copyright (C) 2008-2019 by Jin-Hwan Cho, Matthias Franz, and Shunsaku Hirata,
     the dvipdfmx project team.
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
@@ -1780,7 +1780,6 @@ pdf_doc_add_annot (unsigned page_no, const pdf_rect *rect,
   pdf_doc  *p = &pdoc;
   pdf_page *page;
   pdf_obj  *rect_array;
-  double    annot_grow = p->opt.annot_grow;
   double    xpos, ypos;
   pdf_rect  annbox;
 
@@ -1812,10 +1811,10 @@ pdf_doc_add_annot (unsigned page_no, const pdf_rect *rect,
   }
 
   rect_array = pdf_new_array();
-  pdf_add_array(rect_array, pdf_new_number(ROUND(annbox.llx - annot_grow, 0.001)));
-  pdf_add_array(rect_array, pdf_new_number(ROUND(annbox.lly - annot_grow, 0.001)));
-  pdf_add_array(rect_array, pdf_new_number(ROUND(annbox.urx + annot_grow, 0.001)));
-  pdf_add_array(rect_array, pdf_new_number(ROUND(annbox.ury + annot_grow, 0.001)));
+  pdf_add_array(rect_array, pdf_new_number(ROUND(annbox.llx, 0.001)));
+  pdf_add_array(rect_array, pdf_new_number(ROUND(annbox.lly, 0.001)));
+  pdf_add_array(rect_array, pdf_new_number(ROUND(annbox.urx, 0.001)));
+  pdf_add_array(rect_array, pdf_new_number(ROUND(annbox.ury, 0.001)));
   pdf_add_dict (annot_dict, pdf_new_name("Rect"), rect_array);
 
   pdf_add_array(page->annots, pdf_ref_obj(annot_dict));
@@ -2491,13 +2490,14 @@ pdf_doc_add_page_content (const char *buffer, unsigned length)
 }
 
 void
-pdf_open_document (const char *filename, const char **ids, struct pdf_setting settings)
+pdf_open_document (const char *filename,
+                   const char *creator, const unsigned char *id1, const unsigned char *id2,
+                   struct pdf_setting settings)
 {
   pdf_doc *p = &pdoc;
 
-  pdf_enc_compute_id_string(ids[0], ids[1], ids[2]);
   if (settings.enable_encrypt)
-    pdf_init_encryption(settings.encrypt);
+    pdf_init_encryption(settings.encrypt, id1);
 
   pdf_out_init(filename, settings.enable_encrypt,
                settings.object.enable_objstm, settings.object.enable_predictor);
@@ -2515,10 +2515,10 @@ pdf_open_document (const char *filename, const char **ids, struct pdf_setting se
   pdf_init_images();
 
   pdf_doc_init_docinfo(p);
-  if (ids[3]) {
+  if (creator) {
     pdf_add_dict(p->info,
                  pdf_new_name("Creator"),
-                 pdf_new_string(ids[3], strlen(ids[3])));
+                 pdf_new_string(creator, strlen(creator)));
   }
 
   pdf_doc_init_bookmarks(p, settings.outline_open_depth);
@@ -2533,7 +2533,13 @@ pdf_open_document (const char *filename, const char **ids, struct pdf_setting se
     pdf_set_encrypt(encrypt);
     pdf_release_obj(encrypt);
   }
-  pdf_set_id(pdf_enc_id_array());
+  if (id1 && id2) {
+    pdf_obj *id_obj = pdf_new_array();
+
+    pdf_add_array(id_obj, pdf_new_string(id1, 16));
+    pdf_add_array(id_obj, pdf_new_string(id2, 16));
+    pdf_set_id(id_obj);
+  }
 
   /* Create a default name for thumbnail image files */
   if (manual_thumb_enabled) {
@@ -2795,13 +2801,22 @@ pdf_doc_end_annot (void)
 void
 pdf_doc_break_annot (void)
 {
+  pdf_doc *p = &pdoc;
+  double   g = p->opt.annot_grow;
+
   if (breaking_state.dirty) {
     pdf_obj  *annot_dict;
+    pdf_rect  rect;
 
     /* Copy dict */
     annot_dict = pdf_new_dict();
     pdf_merge_dict(annot_dict, breaking_state.annot_dict);
-    pdf_doc_add_annot(pdf_doc_current_page_number(), &(breaking_state.rect),
+    rect = breaking_state.rect;
+    rect.llx -= g;
+    rect.lly -= g;
+    rect.urx += g;
+    rect.ury += g;
+    pdf_doc_add_annot(pdf_doc_current_page_number(), &rect,
                       annot_dict, !breaking_state.broken);
     pdf_release_obj(annot_dict);
 
