@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2007-2016 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2007-2018 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
@@ -24,6 +24,7 @@
 #define _PDFXIMAGE_H_
 
 #include "pdfdev.h"
+#include "pdfdoc.h"
 
 #define PDF_XOBJECT_TYPE_FORM  0
 #define PDF_XOBJECT_TYPE_IMAGE 1
@@ -50,15 +51,13 @@ typedef struct {
 } xform_info;
 
 typedef struct {
-  int      page_no;
-  int      bbox_type;
+  int  page_no;
+  enum pdf_page_boundary bbox_type;
   pdf_obj *dict;
 } load_options;
 
 typedef struct pdf_ximage_ pdf_ximage;
 
-extern void     pdf_ximage_set_verbose    (void);
-
 extern void     pdf_init_images           (void);
 extern void     pdf_close_images          (void);
 
@@ -102,6 +101,5 @@ pdf_ximage_set_attr (int xobj_id,
                      double llx, double lly, double urx, double ury);
 
 /* Migrated from pdfobj.h. Those are not PDF object related... */
-#define MAX_IMAGES 5000 /* This may be enough */
 
 #endif /* _PDFXIMAGE_H_ */
