@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2002-2014 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2002-2016 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
@@ -23,7 +23,7 @@
 #ifndef _SPC_COLOR_H_
 #define _SPC_COLOR_H_
 
-extern int spc_color_check_special (const char *buffer, long size);
+extern int spc_color_check_special (const char *buffer, int size);
 extern int spc_color_setup_handler (struct spc_handler *handle,
 				    struct spc_env *spe, struct spc_arg *args);
 
