@@ -2,7 +2,7 @@
 ** Directory.cpp                                                        **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,13 +18,12 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
 #include "Directory.hpp"
 
 using namespace std;
 
 #ifdef _WIN32
-	#include <windows.h>
+	#include "windows.hpp"
 #else
 	#include <errno.h>
 	#include <sys/stat.h>
@@ -111,7 +110,7 @@ const char* Directory::read (EntryType type) {
 #else
 	if (_dir) {
 		while ((_dirent = readdir(_dir))) {
-			string path = string(_dirname) + "/" + _dirent->d_name;
+			string path = _dirname + "/" + _dirent->d_name;
 			struct stat stats;
 			if (stat(path.c_str(), &stats) == 0) {
 				if (S_ISDIR(stats.st_mode)) {
