@@ -221,7 +221,7 @@ struct hanzi_index HZ_RADICAL[]={
   {{0x2FD3, L'\0'}, {0xFDD0, 0x9F8D, L'\0'}}, /* 龍 */
   {{0x2FD4, L'\0'}, {0xFDD0, 0x9F9C, L'\0'}}, /* 龜 */
   {{0x2FD5, L'\0'}, {0xFDD0, 0x9FA0, L'\0'}}, /* 龠 */
-  {0, 0},
+  {{L'\0'}, {L'\0'}},
 };
 
 /*
@@ -272,7 +272,7 @@ struct hanzi_index HZ_STROKE[]={
   //  {{0x4E09, 0x5341, 0x516B, 0x756B, L'\0'}, {0xFDD0, 0x2826, L'\0'}}, /* 三十八畫 */
   {{0x4E09, 0x5341, 0x4E5D, 0x756B, L'\0'}, {0xFDD0, 0x2827, L'\0'}}, /* 三十九畫 */
   {{0x56DB, 0x5341, 0x516B, 0x756B, L'\0'}, {0xFDD0, 0x2830, L'\0'}}, /* 四十八畫 */
-  {0, 0},
+  {{L'\0'}, {L'\0'}},
 };
 
 /*
@@ -306,7 +306,7 @@ struct hanzi_index HZ_PINYIN[]={
   {{L'X', L'\0'}, {0xFDD0, L'X', L'\0'}}, /* X */
   {{L'Y', L'\0'}, {0xFDD0, L'Y', L'\0'}}, /* Y */
   {{L'Z', L'\0'}, {0xFDD0, L'Z', L'\0'}}, /* Z */
-  {0, 0},
+  {{L'\0'}, {L'\0'}},
 };
 
 /*
@@ -354,10 +354,10 @@ struct hanzi_index HZ_ZHUYIN[]={
   {{0x3127, L'\0'}, {0xFDD0, 0x3127, L'\0'}}, /* ㄧ */
   {{0x3128, L'\0'}, {0xFDD0, 0x3128, L'\0'}}, /* ㄨ */
   {{0x3129, L'\0'}, {0xFDD0, 0x3129, L'\0'}}, /* ㄩ */
-  {0, 0},
+  {{L'\0'}, {L'\0'}},
 };
 
 struct hanzi_index HZ_UNKNOWN[]={
   {{0x6F22, 0x5B57, L'\0'}, {L'\0'}}, /* 漢字 */
-  {0, 0},
+  {{L'\0'}, {L'\0'}},
 };
