@@ -1063,7 +1063,7 @@ void
 fatal(s,t)
 char*s,*t;
 {
-if(*s)printf("%s",s);
+if(*s)printf(s);
 err_print(t);
 history= fatal_message;exit(wrap_up());
 }
