@@ -7,11 +7,11 @@
 #include <eptexdir/eptex_version.h> /* for EPTEX_VERSION */
 #include <ptexdir/ptex_version.h> /* for PTEX_VERSION */
 
-#define BANNER "This is e-pTeX, Version 3.1415926-" PTEX_VERSION "-" EPTEX_VERSION "-" ETEX_VERSION
+#define BANNER "This is e-pTeX, Version 3.14159265-" PTEX_VERSION "-" EPTEX_VERSION "-" ETEX_VERSION
 #define COPYRIGHT_HOLDER "D.E. Knuth"
 #define AUTHOR "Peter Breitenlohner"
 #define PROGRAM_HELP EPTEXHELP
-#define BUG_ADDRESS "tex-k@tug.org"
+#define BUG_ADDRESS "issue@texjp.org"
 #define DUMP_VAR TEXformatdefault
 #define DUMP_LENGTH_VAR formatdefaultlength
 #define DUMP_OPTION "fmt"
