@@ -2,6 +2,13 @@
 #ifndef PP_CONF_H
 #define PP_CONF_H
 
+//#include "utilarm.h" // keep in sync
+#if defined __arm__ || defined __ARM__ || defined ARM || defined __ARM || defined __arm || defined __ARM_ARCH ||defined __aarch64__ ||( defined(__sun) && defined(__SVR4))
+#  define ARM_COMPLIANT 1
+#else
+#  define ARM_COMPLIANT 0
+#endif
+
 /*
 Aux flags:
   PPDLL -- indicates a part of a shared library
@@ -73,4 +80,4 @@ Aux flags:
 
 #define PPSIZEF PPUINTF
 
-#endif
\ No newline at end of file
+#endif
