@@ -15,7 +15,9 @@
    License for more details.
 
    You should have received a copy of the GNU Lesser General Public License along
-   with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
+   with LuaTeX; if not, see <http://www.gnu.org/licenses/>.
+
+*/
 
 #include <w2c/config.h>
 #include <stdlib.h>
@@ -23,7 +25,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <math.h> /* temporary */
+#include <math.h>
 
 #ifndef pdfTeX
 #  include <lua.h>
@@ -41,20 +43,23 @@
 #define luaL_reg luaL_Reg
 #endif
 
-
 #ifndef lua_objlen
 #define lua_objlen lua_rawlen
 #endif
 
-
 #include "mplib.h"
 #include "mplibps.h"
 #include "mplibsvg.h"
 #include "mplibpng.h"
 
-int luaopen_mplib(lua_State * L); /* forward */
+int luaopen_mplib(lua_State * L);
 
-/* metatable identifiers and tests */
+/*tex
+
+    We need a few metatable identifiers in order to access the metatables for the
+    main object and result userdata.
+
+*/
 
 #define MPLIB_METATABLE     "MPlib.meta"
 #define MPLIB_FIG_METATABLE "MPlib.fig"
@@ -64,25 +69,33 @@ int luaopen_mplib(lua_State * L); /* forward */
 #define is_fig(L,b) (struct mp_edge_object **)luaL_checkudata(L,b,MPLIB_FIG_METATABLE)
 #define is_gr_object(L,b) (struct mp_graphic_object **)luaL_checkudata(L,b,MPLIB_GR_METATABLE)
 
-/* Lua string pre-hashing */
+/*tex
 
-#define mplib_init_S(a) do {                                            \
-    lua_pushliteral(L,#a);                                              \
-    mplib_##a##_ptr = lua_tostring(L,-1);				\
-    mplib_##a##_index = luaL_ref (L,LUA_REGISTRYINDEX);                 \
-  } while (0)
+    We pre-hash the \LUA\ strings which is much faster. The approach is similar to the one
+    used at the \TEX\ end.
 
-#define mplib_push_S(a) do {                                    \
-    lua_rawgeti(L,LUA_REGISTRYINDEX,mplib_##a##_index);         \
-  } while (0)
+*/
 
-#define mplib_is_S(a,i) (mplib_##a##_ptr==lua_tostring(L,i))
+#define mplib_init_S(a) do {                            \
+    lua_pushliteral(L,#a);                              \
+    mplib_##a##_ptr = lua_tostring(L,-1);               \
+    mplib_##a##_index = luaL_ref (L,LUA_REGISTRYINDEX); \
+} while (0)
 
-#define mplib_make_S(a)                                                 \
-  static int mplib_##a##_index = 0;                                     \
-  static const char *mplib_##a##_ptr = NULL
+#define mplib_push_S(a) do {                            \
+    lua_rawgeti(L,LUA_REGISTRYINDEX,mplib_##a##_index); \
+} while (0)
 
-static int mplib_type_Ses[mp_special_code + 1] = { 0 }; /* [0] is not used */
+#define mplib_is_S(a,i) \
+    (mplib_##a##_ptr==lua_tostring(L,i))
+
+#define mplib_make_S(a)                     \
+    static int mplib_##a##_index = 0;         \
+    static const char *mplib_##a##_ptr = NULL
+
+/*tex In the next array entry 0 is not used */
+
+static int mplib_type_Ses[mp_special_code + 1] = { 0 };
 
 mplib_make_S(term);
 mplib_make_S(error);
@@ -94,6 +107,7 @@ mplib_make_S(memory);
 mplib_make_S(hash);
 mplib_make_S(params);
 mplib_make_S(open);
+mplib_make_S(cycle);
 
 mplib_make_S(offset);
 mplib_make_S(dashes);
@@ -153,6 +167,7 @@ static void mplib_init_Ses(lua_State * L)
     mplib_init_S(hash);
     mplib_init_S(params);
     mplib_init_S(open);
+    mplib_init_S(cycle);
 
     mplib_init_S(offset);
     mplib_init_S(dashes);
@@ -210,11 +225,15 @@ static void mplib_init_Ses(lua_State * L)
     mplib_init_S(elliptical);
 }
 
+/*tex
 
-/* Enumeration arrays to map MPlib enums to Lua strings */
+    Here are some enumeration arrays to map MPlib enums to \LUA\ strings. If needed
+    we can also predefine keys here, as we do with nodes.
+
+*/
 
 static const char *math_options[] =
-  { "scaled", "double", "binary", "decimal", NULL };
+    { "scaled", "double", "binary", "decimal", NULL };
 
 static const char *interaction_options[] =
     { "unknown", "batch", "nonstop", "scroll", "errorstop", NULL };
@@ -255,18 +274,34 @@ static const char *stop_clip_fields[] =
 static const char *no_fields[] =
     { NULL };
 
+/*tex
 
-/* The list of supported MPlib options (not all make sense) */
+    The list of supported MPlib options (not all make sense).
+
+*/
 
 typedef enum {
-    P_ERROR_LINE, P_MAX_LINE, P_RANDOM_SEED, P_MATH_MODE,
-    P_INTERACTION, P_INI_VERSION, P_MEM_NAME, P_JOB_NAME, P_FIND_FILE,
-    P_RUN_SCRIPT, P_MAKE_TEXT, P_SCRIPT_ERROR, P_EXTENSIONS,
-    P__SENTINEL } mplib_parm_idx;
+    P_ERROR_LINE,
+    P_MAX_LINE,
+    P_RANDOM_SEED,
+    P_MATH_MODE,
+    P_INTERACTION,
+    P_INI_VERSION,
+    P_MEM_NAME,
+    P_JOB_NAME,
+    P_FIND_FILE,
+    P_RUN_SCRIPT,
+    P_MAKE_TEXT,
+    P_SCRIPT_ERROR,
+    P_EXTENSIONS,
+    P__SENTINEL
+} mplib_parm_idx;
 
 typedef struct {
-    const char *name;           /* parameter name */
-    mplib_parm_idx idx;         /* parameter index */
+    /*tex parameter name */
+    const char *name;
+    /*tex parameter index */
+    mplib_parm_idx idx;
 } mplib_parm_struct;
 
 static mplib_parm_struct mplib_parms[] = {
@@ -284,10 +319,11 @@ static mplib_parm_struct mplib_parms[] = {
     {NULL,           P__SENTINEL    }
 };
 
+/*tex
 
-/* Start by defining the needed callback routines for the library  */
+    We start by defining the needed callback routines for the library.
 
-/* todo: make subtable in registry, beware, for all mp instances */
+*/
 
 static char *mplib_find_file(MP mp, const char *fname, const char *fmode, int ftype)
 {
@@ -300,7 +336,7 @@ static char *mplib_find_file(MP mp, const char *fname, const char *fmode, int ft
         lua_pushstring(L, fname);
         lua_pushstring(L, fmode);
         if (ftype >= mp_filetype_text) {
-          lua_pushnumber(L, (lua_Number)(ftype - mp_filetype_text));
+            lua_pushinteger(L, (ftype - mp_filetype_text));
         } else {
             lua_pushstring(L, mplib_filetype_names[ftype]);
         }
@@ -311,7 +347,8 @@ static char *mplib_find_file(MP mp, const char *fname, const char *fmode, int ft
         x = lua_tostring(L, -1);
         if (x != NULL)
             s = strdup(x);
-        lua_pop(L, 1);          /* pop the string */
+        /*tex pop the string */
+        lua_pop(L, 1);
         return s;
     } else {
         lua_pop(L, 1);
@@ -325,7 +362,8 @@ static char *mplib_find_file(MP mp, const char *fname, const char *fmode, int ft
 static int mplib_find_file_function(lua_State * L)
 {
     if (!(lua_isfunction(L, -1) || lua_isnil(L, -1))) {
-        return 1;               /* error */
+        /*tex An error. */
+        return 1;
     }
     lua_pushstring(L, "mplib.file_finder");
     lua_pushvalue(L, -2);
@@ -333,6 +371,11 @@ static int mplib_find_file_function(lua_State * L)
     return 0;
 }
 
+static void mplib_warning(const char *str)
+{
+    fprintf(stdout,"mplib warning: %s\n",str);
+}
+
 static void mplib_script_error(MP mp, const char *str)
 {
     lua_State *L = (lua_State *)mp_userdata(mp);
@@ -340,9 +383,10 @@ static void mplib_script_error(MP mp, const char *str)
     lua_getfield(L, LUA_REGISTRYINDEX, "mplib.script_error");
     if (lua_isfunction(L, -1)) {
         lua_pushstring(L, str);
-        lua_pcall(L, 1, 0, 0); /* assume the function is ok */
+        /*tex We assume that the function is okay. */
+        lua_pcall(L, 1, 0, 0);
     } else {
-        fprintf(stdout,"Error in script: %s\n",str);
+        mplib_warning(str);
         lua_pop(L, 1);
     }
 }
@@ -350,7 +394,8 @@ static void mplib_script_error(MP mp, const char *str)
 static int mplib_script_error_function(lua_State * L)
 {
     if (!(lua_isfunction(L, -1) || lua_isnil(L, -1))) {
-        return 1;               /* error */
+        /*tex An error. */
+        return 1;
     }
     lua_pushstring(L, "mplib.script_error");
     lua_pushvalue(L, -2);
@@ -368,13 +413,14 @@ static char *mplib_run_script(MP mp, const char *str)
         const char *x = NULL;
         lua_pushstring(L, str);
         if (lua_pcall(L, 1, 1, 0) != 0) {
-            mplib_script_error(mp, lua_tostring(L, -1));
+            fprintf(stdout,"mplib warning: error in script: %s\n",lua_tostring(L, -1));
             return NULL;
         }
         x = lua_tostring(L, -1);
         if (x != NULL)
             s = strdup(x);
-        lua_pop(L, 1);          /* pop the string */
+        /*tex Pop the string. */
+        lua_pop(L, 1);
         return s;
     } else {
         lua_pop(L, 1);
@@ -385,7 +431,7 @@ static char *mplib_run_script(MP mp, const char *str)
 static int mplib_run_script_function(lua_State * L)
 {
     if (!(lua_isfunction(L, -1) || lua_isnil(L, -1))) {
-        return 1;               /* error */
+        return 1; /* error */
     }
     lua_pushstring(L, "mplib.run_script");
     lua_pushvalue(L, -2);
@@ -410,7 +456,8 @@ static char *mplib_make_text(MP mp, const char *str, int mode)
         x = lua_tostring(L, -1);
         if (x != NULL)
             s = strdup(x);
-        lua_pop(L, 1);          /* pop the string */
+        /*tex Pop the string. */
+        lua_pop(L, 1);
         return s;
     } else {
         lua_pop(L, 1);
@@ -421,7 +468,8 @@ static char *mplib_make_text(MP mp, const char *str, int mode)
 static int mplib_make_text_function(lua_State * L)
 {
     if (!(lua_isfunction(L, -1) || lua_isnil(L, -1))) {
-        return 1;               /* error */
+        /*tex An error. */
+        return 1;
     }
     lua_pushstring(L, "mplib.make_text");
     lua_pushvalue(L, -2);
@@ -487,7 +535,8 @@ static int mplib_new(lua_State * L)
         int i;
         struct MP_options *options = mp_options();
         options->userdata = (void *) L;
-        options->noninteractive = 1;    /* required ! */
+        /*tex Required: */
+        options->noninteractive = 1;
         options->extensions = 0 ;
         options->find_file = mplib_find_file;
         options->run_script = mplib_run_script;
@@ -500,56 +549,59 @@ static int mplib_new(lua_State * L)
                 lua_getfield(L, 1, mplib_parms[i].name);
                 if (lua_isnil(L, -1)) {
                     lua_pop(L, 1);
-                    continue;   /* skip unset */
+                    continue;
                 }
                 switch (mplib_parms[i].idx) {
-                case P_ERROR_LINE:
-                  options->error_line = (int)lua_tointeger(L, -1);
-                    if (options->error_line<60) options->error_line =60;
-                    if (options->error_line>250) options->error_line = 250;
-                    options->half_error_line = (options->error_line/2)+10;
-                    break;
-                case P_MAX_LINE:
-                    options->max_print_line = (int)lua_tointeger(L, -1);
-                    if (options->max_print_line<60) options->max_print_line = 60;
-                    break;
-                case P_RANDOM_SEED:
-                    options->random_seed = (int)lua_tointeger(L, -1);
-                    break;
-                case P_INTERACTION:
-                    options->interaction = luaL_checkoption(L, -1, "errorstopmode", interaction_options);
-                    break;
-                case P_MATH_MODE:
-                    options->math_mode = luaL_checkoption(L, -1, "scaled", math_options);
-                    break;
-                case P_JOB_NAME:
-                    options->job_name = strdup(lua_tostring(L, -1));
-                    break;
-                case P_FIND_FILE:
-                    if (mplib_find_file_function(L)) {  /* error here */
-                        fprintf(stdout,"Invalid arguments to mp.new { find_file = ... }\n");
-                    }
-                    break;
-                case P_RUN_SCRIPT:
-                    if (mplib_run_script_function(L)) {  /* error here */
-                        fprintf(stdout,"Invalid arguments to mp.new { run_script = ... }\n");
-                    }
-                    break;
-                case P_MAKE_TEXT:
-                    if (mplib_make_text_function(L)) {  /* error here */
-                        fprintf(stdout,"Invalid arguments to mp.new { make_text = ... }\n");
-                    }
-                    break;
-                case P_SCRIPT_ERROR:
-                    if (mplib_script_error_function(L)) {  /* error here */
-                        fprintf(stdout,"Invalid arguments to mp.new { script_error = ... }\n");
-                    }
-                    break;
-                case P_EXTENSIONS:
-                    options->extensions = (int)lua_tointeger(L, -1);
-                    break;
-                default:
-                    break;
+                    case P_ERROR_LINE:
+                        options->error_line = (int)lua_tointeger(L, -1);
+                        if (options->error_line < 60)
+                            options->error_line = 60;
+                        if (options->error_line > 250)
+                            options->error_line = 250;
+                        options->half_error_line = (options->error_line/2)+10;
+                        break;
+                    case P_MAX_LINE:
+                        options->max_print_line = (int)lua_tointeger(L, -1);
+                        if (options->max_print_line < 60)
+                            options->max_print_line = 60;
+                        break;
+                    case P_RANDOM_SEED:
+                        options->random_seed = (int)lua_tointeger(L, -1);
+                        break;
+                    case P_INTERACTION:
+                        options->interaction = luaL_checkoption(L, -1, "errorstopmode", interaction_options);
+                        break;
+                    case P_MATH_MODE:
+                        options->math_mode = luaL_checkoption(L, -1, "scaled", math_options);
+                        break;
+                    case P_JOB_NAME:
+                        options->job_name = strdup(lua_tostring(L, -1));
+                        break;
+                    case P_FIND_FILE:
+                        if (mplib_find_file_function(L)) {
+                            mplib_warning("function expected for 'find_file'");
+                        }
+                        break;
+                    case P_RUN_SCRIPT:
+                        if (mplib_run_script_function(L)) {
+                            mplib_warning("function expected for 'run_script'");
+                        }
+                        break;
+                    case P_MAKE_TEXT:
+                        if (mplib_make_text_function(L)) {
+                            mplib_warning("function expected for 'make_text'");
+                        }
+                        break;
+                    case P_SCRIPT_ERROR:
+                        if (mplib_script_error_function(L)) {
+                            mplib_warning("function expected for 'script_error'");
+                        }
+                        break;
+                    case P_EXTENSIONS:
+                        options->extensions = (int)lua_tointeger(L, -1);
+                        break;
+                    default:
+                        break;
                 }
                 lua_pop(L, 1);
             }
@@ -626,7 +678,7 @@ static int mplib_wrapresults(lua_State * L, mp_run_data *res, int status)
         res->edges = NULL;
     }
     mplib_push_S(status);
-    lua_pushnumber(L, (lua_Number)status);
+    lua_pushinteger(L, status);
     lua_rawset(L,-3);
     return 1;
 }
@@ -716,16 +768,16 @@ static int mplib_statistics(lua_State * L)
     if (*mp_ptr != NULL) {
         lua_newtable(L);
         mplib_push_S(memory);
-        lua_pushnumber(L, (lua_Number)mp_memory_usage(*mp_ptr));
+        lua_pushinteger(L, mp_memory_usage(*mp_ptr));
         lua_rawset(L,-3);
         mplib_push_S(hash);
-        lua_pushnumber(L, (lua_Number)mp_hash_usage(*mp_ptr));
+        lua_pushinteger(L, mp_hash_usage(*mp_ptr));
         lua_rawset(L,-3);
         mplib_push_S(params);
-        lua_pushnumber(L, (lua_Number)mp_param_usage(*mp_ptr));
+        lua_pushinteger(L, mp_param_usage(*mp_ptr));
         lua_rawset(L,-3);
         mplib_push_S(open);
-        lua_pushnumber(L, (lua_Number)mp_open_usage(*mp_ptr));
+        lua_pushinteger(L, mp_open_usage(*mp_ptr));
         lua_rawset(L,-3);
     } else {
         lua_pushnil(L);
@@ -733,7 +785,6 @@ static int mplib_statistics(lua_State * L)
     return 1;
 }
 
-
 static int set_direction (lua_State * L, MP mp, mp_knot p) {
     double direction_x = 0, direction_y = 0;
     direction_x = (double)lua_tonumber(L,-1);
@@ -821,8 +872,8 @@ static int set_right_control (lua_State * L, MP mp, mp_knot p) {
     return 1;
 }
 
-
 #if 0
+
 #define ROUNDED_ZERO(v) (fabs((v))<0.00001 ? 0 : (v))
 #define PI 3.1415926535897932384626433832795028841971
 #define RADIANS(a) (mp_number_as_double(mp,(a)) / 16.0) * PI/180.0
@@ -915,6 +966,7 @@ void mp_dump_path (MP mp, mp_knot h) {
         printf("cycle");
     printf (";\n");
 }
+
 #endif
 
 static int mplib_solve_path(lua_State * L)
@@ -938,10 +990,8 @@ static int mplib_solve_path(lua_State * L)
     mp = *mp_ptr;
     cyclic = lua_toboolean(L,3);
     lua_pop(L,1);
-
-    /* build up the path */
+    /*tex We build up the path. */
     numpoints = lua_objlen(L,2);
-
     first = p = NULL;
     for (i=1;i<=numpoints;i++) {
         int left_set = 0, right_set = 0;
@@ -951,7 +1001,6 @@ static int mplib_solve_path(lua_State * L)
             errormsg = "Wrong argument types";
             goto BAD;
         }
-
         mplib_push_S(x_coord);
         lua_rawget(L,-2);
         if (!lua_isnumber(L,-1)) {
@@ -960,7 +1009,6 @@ static int mplib_solve_path(lua_State * L)
         }
         x_coord = (double)lua_tonumber(L,-1);
         lua_pop(L,1);
-
         mplib_push_S(y_coord);
         lua_rawget(L,-2);
         if (!lua_isnumber(L,-1)) {
@@ -969,11 +1017,14 @@ static int mplib_solve_path(lua_State * L)
         }
         y_coord = (double)lua_tonumber(L,-1);
         lua_pop(L,1);
-
         q = p;
         if (q!=NULL) {
-            /* we have to save the right_tension because |mp_append_knot| trashes
-               it, believing that it is as yet uninitialized */
+            /*tex
+
+                We have to save the right_tension because |mp_append_knot|
+                trashes it, believing that it is as yet uninitialized.
+
+            */
             double saved_tension = mp_number_as_double(mp, mp_knot_right_tension(mp,p));
             p = mp_append_knot(mp, p, x_coord, y_coord);
             if ( ! p ) {
@@ -988,10 +1039,8 @@ static int mplib_solve_path(lua_State * L)
                 goto BAD;
             }
         }
-
         if (first == NULL)
             first = p;
-
         mplib_push_S(left_curl);
         lua_rawget(L,-2);
         if (lua_isnumber(L,-1)) {
@@ -1001,9 +1050,9 @@ static int mplib_solve_path(lua_State * L)
             }
             left_set  = 1;
         } else {
-            lua_pop(L,1); /* a nil value */
+            /*tex A |nil| value. */
+            lua_pop(L,1);
         }
-
         mplib_push_S(left_tension);
         lua_rawget(L,-2);
         if (lua_isnumber(L,-1)) {
@@ -1018,9 +1067,9 @@ static int mplib_solve_path(lua_State * L)
                 left_set  = 1;
             }
         } else {
-            lua_pop(L,1); /* a nil value */
+            /*tex A |nil| value. */
+            lua_pop(L,1);
         }
-
         mplib_push_S(left_x);
         lua_rawget(L,-2);
         if (lua_isnumber(L,-1)) {
@@ -1036,7 +1085,6 @@ static int mplib_solve_path(lua_State * L)
         } else {
             lua_pop(L,1);
         }
-
         mplib_push_S(right_curl);
         lua_rawget(L,-2);
         if (lua_isnumber(L,-1)) {
@@ -1046,9 +1094,9 @@ static int mplib_solve_path(lua_State * L)
             }
             right_set  = 1;
         } else {
-            lua_pop(L,1); /* a nil value */
+            /*tex A |nil| value. */
+            lua_pop(L,1);
         }
-
         mplib_push_S(right_tension);
         lua_rawget(L,-2);
         if (lua_isnumber(L,-1)) {
@@ -1065,7 +1113,6 @@ static int mplib_solve_path(lua_State * L)
         } else {
             lua_pop(L,1);
         }
-
         mplib_push_S(right_x);
         lua_rawget(L,-2);
         if (lua_isnumber(L,-1)) {
@@ -1081,7 +1128,6 @@ static int mplib_solve_path(lua_State * L)
         } else {
             lua_pop(L,1);
         }
-
         mplib_push_S(direction_x);
         lua_rawget(L,-2);
         if (lua_isnumber(L,-1)) {
@@ -1090,12 +1136,12 @@ static int mplib_solve_path(lua_State * L)
                 goto BAD;
             }
         } else {
-            lua_pop(L,1); /* a nil value */
+            /*tex A |nil| value. */
+            lua_pop(L,1);
         }
-
-        lua_pop(L,1); /* done with this item */
+        /*tex Up the next item */
+        lua_pop(L,1);
     }
-
     if (cyclic) {
         mp_close_path_cycle (mp, p, first);
     } else {
@@ -1104,12 +1150,12 @@ static int mplib_solve_path(lua_State * L)
 #if 0
     mp_dump_path(mp,first);
 #endif
-    /* finished reading arguments */
+    /*tex We're finished reading arguments. */
     if (!mp_solve_path(mp,first)) {
         errormsg = "Failed to solve the path";
         goto BAD;
     }
-    /* squeeze the new values back into the table */
+    /*tex Squeeze the new values back into the table. */
     p = first;
     for (i=1;i<=numpoints;i++) {
         lua_rawgeti(L,-1, i);
@@ -1117,14 +1163,14 @@ static int mplib_solve_path(lua_State * L)
         mplib_push_S(left_y);  lua_pushnumber(L, mp_number_as_double(mp, mp_knot_left_y(mp, p)));  lua_rawset(L,-3);
         mplib_push_S(right_x); lua_pushnumber(L, mp_number_as_double(mp, mp_knot_right_x(mp, p))); lua_rawset(L,-3);
         mplib_push_S(right_y); lua_pushnumber(L, mp_number_as_double(mp, mp_knot_right_y(mp, p))); lua_rawset(L,-3);
-        /* is this really needed */
-            mplib_push_S(left_tension);  lua_pushnil(L); lua_rawset(L,-3);
-            mplib_push_S(right_tension); lua_pushnil(L); lua_rawset(L,-3);
-            mplib_push_S(left_curl);     lua_pushnil(L); lua_rawset(L,-3);
-            mplib_push_S(right_curl);    lua_pushnil(L); lua_rawset(L,-3);
-            mplib_push_S(direction_x);   lua_pushnil(L); lua_rawset(L,-3);
-            mplib_push_S(direction_y);   lua_pushnil(L); lua_rawset(L,-3);
-        /* till here */
+        /*tex This is a bit overkill \unknown */
+        mplib_push_S(left_tension);  lua_pushnil(L); lua_rawset(L,-3);
+        mplib_push_S(right_tension); lua_pushnil(L); lua_rawset(L,-3);
+        mplib_push_S(left_curl);     lua_pushnil(L); lua_rawset(L,-3);
+        mplib_push_S(right_curl);    lua_pushnil(L); lua_rawset(L,-3);
+        mplib_push_S(direction_x);   lua_pushnil(L); lua_rawset(L,-3);
+        mplib_push_S(direction_y);   lua_pushnil(L); lua_rawset(L,-3);
+        /*tex \unknown\ till here. */
         mplib_push_S(left_type);  lua_pushstring(L, knot_type_enum[mp_knot_left_type(mp, p)]);  lua_rawset(L, -3);
         mplib_push_S(right_type); lua_pushstring(L, knot_type_enum[mp_knot_right_type(mp, p)]); lua_rawset(L, -3);
         lua_pop(L,1);
@@ -1132,7 +1178,7 @@ static int mplib_solve_path(lua_State * L)
     }
     lua_pushboolean(L, 1);
     return 1;
-BAD:
+  BAD:
     if (p != NULL) {
         mp_close_path (mp, p, first);
         mp_free_path (mp, p);
@@ -1142,7 +1188,11 @@ BAD:
     return 2;
 }
 
-/* figure methods */
+/*tex
+
+    The next methods are for collecting the results from |fig|.
+
+*/
 
 static int mplib_fig_collect(lua_State * L)
 {
@@ -1171,7 +1221,8 @@ static int mplib_fig_body(lua_State * L)
         i++;
         p = p->next;
     }
-    (*hh)->body = NULL;         /* prevent double free */
+    /*tex Prevent a double free: */
+    (*hh)->body = NULL;
     return 1;
 }
 
@@ -1233,7 +1284,6 @@ static int mplib_fig_svg(lua_State * L)
     return 1;
 }
 
-
 static int mplib_fig_png(lua_State * L)
 {
     mp_run_data *res;
@@ -1316,8 +1366,6 @@ static int mplib_fig_charcode(lua_State * L)
     return 1;
 }
 
-
-
 static int mplib_fig_bb(lua_State * L)
 {
     struct mp_edge_object **hh = is_fig(L, 1);
@@ -1333,7 +1381,11 @@ static int mplib_fig_bb(lua_State * L)
     return 1;
 }
 
-/* object methods */
+/*tex
+
+    The methods for the figure objects plus a few helpers.
+
+*/
 
 static int mplib_gr_collect(lua_State * L)
 {
@@ -1413,7 +1465,6 @@ static double coord_range_y (mp_gr_knot h, double dz) {
     return (zhi - zlo <= dz ? aspect_bound : aspect_default);
 }
 
-
 static int mplib_gr_peninfo(lua_State * L) {
     double x_coord, y_coord, left_x, left_y, right_x, right_y;
     double wx, wy;
@@ -1481,6 +1532,13 @@ static int mplib_gr_peninfo(lua_State * L) {
     return 1;
 }
 
+/*tex
+
+    Here is a helper that reports the valid field names of the possible
+    objects.
+
+*/
+
 static int mplib_gr_fields(lua_State * L)
 {
     const char **fields;
@@ -1526,7 +1584,6 @@ static int mplib_gr_fields(lua_State * L)
     return 1;
 }
 
-
 #define mplib_push_number(L,x) lua_pushnumber(L,(lua_Number)(x))
 
 #define MPLIB_PATH 0
@@ -1534,7 +1591,7 @@ static int mplib_gr_fields(lua_State * L)
 
 static void mplib_push_path(lua_State * L, mp_gr_knot h, int is_pen)
 {
-    mp_gr_knot p;          /* for scanning the path */
+    mp_gr_knot p;
     int i = 1;
     p = h;
     if (p != NULL) {
@@ -1583,16 +1640,63 @@ static void mplib_push_path(lua_State * L, mp_gr_knot h, int is_pen)
     }
 }
 
-/* this assumes that the top of the stack is a table
-   or nil already in the case
+static int mplib_get_path(lua_State * L)
+{
+    MP *mp = is_mp(L, 1);
+    if (*mp != NULL) {
+        size_t l;
+        const char *s = lua_tolstring(L, 2, &l);
+        if (s != NULL) {
+            mp_knot p = mp_get_path_value(*mp,s,l) ;
+            if (p != NULL) {
+                int i = 1;
+                mp_knot h = p;
+                lua_newtable(L);
+                do {
+                    lua_createtable(L, 6, 1);
+                    mplib_push_number(L, mp_number_as_double(*mp,p->x_coord));
+                    lua_rawseti(L,-2,1);
+                    mplib_push_number(L, mp_number_as_double(*mp,p->y_coord));
+                    lua_rawseti(L,-2,2);
+                    mplib_push_number(L, mp_number_as_double(*mp,p->left_x));
+                    lua_rawseti(L,-2,3);
+                    mplib_push_number(L, mp_number_as_double(*mp,p->left_y));
+                    lua_rawseti(L,-2,4);
+                    mplib_push_number(L, mp_number_as_double(*mp,p->right_x));
+                    lua_rawseti(L,-2,5);
+                    mplib_push_number(L, mp_number_as_double(*mp,p->right_y));
+                    lua_rawseti(L,-2,6);
+                    lua_rawseti(L,-2, i);
+                    i++;
+                    if (p->data.types.right_type == mp_endpoint) {
+                        mplib_push_S(cycle);
+                        lua_pushboolean(L,0);
+                        lua_rawset(L,-3);
+                        return 1;
+                    }
+                    p = p->next;
+                } while (p != h);
+                mplib_push_S(cycle);
+                lua_pushboolean(L,1);
+                lua_rawset(L,-3);
+                return 1;
+            }
+        }
+    }
+    return 0;
+}
+
+/*tex
+
+    This assumes that the top of the stack is a table or nil already in the case.
 */
 
 static void mplib_push_pentype(lua_State * L, mp_gr_knot h)
 {
-    mp_gr_knot p;          /* for scanning the path */
+    mp_gr_knot p;
     p = h;
     if (p == NULL) {
-        /* do nothing */
+        /*tex Do nothing. */
     } else if (p == p->next) {
         mplib_push_S(type);
         mplib_push_S(elliptical);
@@ -1643,7 +1747,11 @@ static void mplib_push_color(lua_State * L, struct mp_graphic_object *p)
     }
 }
 
-/* the dash scale is not exported, the field has no external value */
+/*tex
+
+    The dash scale is not exported, the field has no external value.
+
+*/
 
 static void mplib_push_dash(lua_State * L, struct mp_stroked_object *h)
 {
@@ -1811,7 +1919,6 @@ static int mplib_gr_index(lua_State * L)
     struct mp_graphic_object **hh = is_gr_object(L, 1);
     if (*hh) {
         struct mp_graphic_object *h = *hh;
-
         if (mplib_is_S(type, 2)) {
             lua_rawgeti(L, LUA_REGISTRYINDEX, mplib_type_Ses[h->type]);
         } else {
@@ -1847,61 +1954,77 @@ static int mplib_gr_index(lua_State * L)
 }
 
 static const struct luaL_reg mplib_meta[] = {
-    {"__gc", mplib_collect},
-    {"__tostring", mplib_tostring},
-    {NULL, NULL} /* sentinel */
+    { "__gc",       mplib_collect },
+    { "__tostring", mplib_tostring },
+    /*tex sentinel */
+    { NULL,         NULL}
 };
 
 static const struct luaL_reg mplib_fig_meta[] = {
-    {"__gc",         mplib_fig_collect},
-    {"__tostring",   mplib_fig_tostring},
-    {"objects",      mplib_fig_body},
-    {"copy_objects", mplib_fig_copy_body},
-    {"filename",     mplib_fig_filename},
-    {"postscript",   mplib_fig_postscript},
-    {"png",          mplib_fig_png},
-    {"svg",          mplib_fig_svg},
-    {"boundingbox",  mplib_fig_bb},
-    {"width",        mplib_fig_width},
-    {"height",       mplib_fig_height},
-    {"depth",        mplib_fig_depth},
-    {"italcorr",     mplib_fig_italcorr},
-    {"charcode",     mplib_fig_charcode},
-    {NULL, NULL}                /* sentinel */
+    { "__gc",         mplib_fig_collect },
+    { "__tostring",   mplib_fig_tostring },
+    { "objects",      mplib_fig_body },
+    { "copy_objects", mplib_fig_copy_body },
+    { "filename",     mplib_fig_filename },
+    { "postscript",   mplib_fig_postscript },
+    { "png",          mplib_fig_png },
+    { "svg",          mplib_fig_svg },
+    { "boundingbox",  mplib_fig_bb },
+    { "width",        mplib_fig_width },
+    { "height",       mplib_fig_height },
+    { "depth",        mplib_fig_depth },
+    { "italcorr",     mplib_fig_italcorr },
+    { "charcode",     mplib_fig_charcode },
+    /*tex sentinel */
+    { NULL,           NULL}
 };
 
 static const struct luaL_reg mplib_gr_meta[] = {
-    {"__gc", mplib_gr_collect},
-    {"__tostring", mplib_gr_tostring},
-    {"__index", mplib_gr_index},
-    {NULL, NULL} /* sentinel */
+    { "__gc",       mplib_gr_collect},
+    { "__tostring", mplib_gr_tostring},
+    { "__index",    mplib_gr_index},
+    /*tex sentinel */
+    { NULL,         NULL}
 };
 
 static const struct luaL_reg mplib_d[] = {
-    {"execute", mplib_execute},
-    {"finish", mplib_finish},
-    {"char_width", mplib_charwidth},
-    {"char_height", mplib_charheight},
-    {"char_depth", mplib_chardepth},
-    {"statistics", mplib_statistics},
-    {"solve_path", mplib_solve_path},
-    {"get_numeric", mplib_get_numeric},
-    {"get_number", mplib_get_numeric},
-    {"get_boolean", mplib_get_boolean},
-    {"get_string", mplib_get_string},
-    {NULL, NULL} /* sentinel */
+    { "execute",     mplib_execute },
+    { "finish",      mplib_finish },
+    { "char_width",  mplib_charwidth },
+    { "char_height", mplib_charheight },
+    { "char_depth",  mplib_chardepth },
+    { "statistics",  mplib_statistics },
+    { "solve_path",  mplib_solve_path },
+    { "get_numeric", mplib_get_numeric },
+    { "get_number",  mplib_get_numeric },
+    { "get_boolean", mplib_get_boolean },
+    { "get_string",  mplib_get_string },
+    { "get_path",    mplib_get_path },
+    /*tex sentinel */
+    {NULL,           NULL }
 };
 
 static const struct luaL_reg mplib_m[] = {
-    {"new", mplib_new},
-    {"version",    mplib_version},
-    {"fields", mplib_gr_fields},
-    {"pen_info", mplib_gr_peninfo},
-    {"get_numeric", mplib_get_numeric},
-    {"get_number", mplib_get_numeric},
-    {"get_boolean", mplib_get_boolean},
-    {"get_string", mplib_get_string},
-    {NULL, NULL} /* sentinel */
+    { "new",         mplib_new },
+    { "version",     mplib_version },
+    { "fields",      mplib_gr_fields },
+    /* indirect */
+    { "execute",     mplib_execute },
+    { "finish",      mplib_finish },
+    { "char_width",  mplib_charwidth },
+    { "char_height", mplib_charheight },
+    { "char_depth",  mplib_chardepth },
+    { "statistics",  mplib_statistics },
+    { "solve_path",  mplib_solve_path },
+    /* helpers */
+    { "pen_info",    mplib_gr_peninfo },
+    { "get_numeric", mplib_get_numeric },
+    { "get_number",  mplib_get_numeric },
+    { "get_boolean", mplib_get_boolean },
+    { "get_string",  mplib_get_string },
+    { "get_path",    mplib_get_path },
+    /*tex sentinel */
+    { NULL,          NULL}
 };
 
 int luaopen_mplib(lua_State * L)
@@ -1909,22 +2032,23 @@ int luaopen_mplib(lua_State * L)
     mplib_init_Ses(L);
 
     luaL_newmetatable(L, MPLIB_GR_METATABLE);
-    lua_pushvalue(L, -1);                       /* push metatable */
-    lua_setfield(L, -2, "__index");             /* metatable.__index = metatable */
-    luaL_register(L, NULL, mplib_gr_meta);      /* object meta methods */
+    lua_pushvalue(L, -1);
+    lua_setfield(L, -2, "__index");
+    luaL_register(L, NULL, mplib_gr_meta);
     lua_pop(L, 1);
 
     luaL_newmetatable(L, MPLIB_FIG_METATABLE);
-    lua_pushvalue(L, -1);                       /* push metatable */
-    lua_setfield(L, -2, "__index");             /* metatable.__index = metatable */
-    luaL_register(L, NULL, mplib_fig_meta);     /* figure meta methods */
+    lua_pushvalue(L, -1);
+    lua_setfield(L, -2, "__index");
+    luaL_register(L, NULL, mplib_fig_meta);
     lua_pop(L, 1);
 
     luaL_newmetatable(L, MPLIB_METATABLE);
-    lua_pushvalue(L, -1);                       /* push metatable */
-    lua_setfield(L, -2, "__index");             /* metatable.__index = metatable */
-    luaL_register(L, NULL, mplib_meta);         /* meta methods */
-    luaL_register(L, NULL, mplib_d);            /* dict methods */
-    luaL_register(L, "mplib", mplib_m);         /* module functions */
+    lua_pushvalue(L, -1);
+    lua_setfield(L, -2, "__index");
+    luaL_register(L, NULL, mplib_meta);
+    luaL_register(L, NULL, mplib_d);
+    luaL_register(L, "mplib", mplib_m);
+
     return 1;
 }
