@@ -39,6 +39,7 @@ extern integer calc_pos (integer c);
 #define calcpos calc_pos
 extern integer kcatcodekey (integer c);
 
+extern void init_kanji (const_string file_str, const_string internal_str);
 extern void init_default_kanji (const_string file_str, const_string internal_str);
 #ifdef PBIBTEX
 /* pBibTeX is EUC only */
@@ -57,10 +58,20 @@ extern void init_default_kanji (const_string file_str, const_string internal_str
 #define putc(c,fp) putc2(c,fp)
 #endif /* !PRESERVE_PUTC */
 
+#ifndef PRESERVE_FPUTS
+#undef fputs
+#define fputs(c,fp) fputs2(c,fp)
+#endif /* !PRESERVE_FPUTS */
+
 #ifdef PBIBTEX
 #define inputline2(fp,buff,pos,size,ptr) input_line2(fp,buff,pos,size,ptr)
 #else
 #define inputline2(fp,buff,pos,size) input_line2(fp,buff,pos,size,NULL)
 #endif
 
+extern void dump_kanji (FILE *fp);
+extern void undump_kanji (FILE *fp);
+#define dumpkanji dump_kanji
+#define undumpkanji undump_kanji
+
 #endif /* not KANJI_H */
