@@ -27,7 +27,7 @@ boolean is_char_ascii(integer c)
 boolean is_char_kanji(integer c)
 {
     if (is_internalUPTEX()) 
-        return (c>=0&&(c & CJK_TOKEN_FLAG) < CJK_CHAR_LIMIT);
+        return (c >= 0);
     else
         return iskanji1(Hi(c)) && iskanji2(Lo(c));
 }
@@ -56,25 +56,25 @@ integer calc_pos(integer c)
 }
 
 /* Ref. http://www.unicode.org/Public/UNIDATA/Blocks.txt */
-/* # Blocks-10.0.0.txt                                   */
-/* # Date: 2017-04-12, 17:30:00 GMT [KW]                 */
+/* # Blocks-12.0.0.txt                                   */
+/* # Date: 2018-07-30, 19:40:00 GMT [KW]                 */
 static long ucs_range[]={
       0x0000, /* Basic Latin					     */ /* 0x00 */
       0x0080, /* Latin-1 Supplement				     */
       0x0100, /* Latin Extended-A				     */
       0x0180, /* Latin Extended-B				     */
-      0x0250, /* IPA Extensions					     */
+      0x0250, /* IPA Extensions				     */
       0x02B0, /* Spacing Modifier Letters			     */
       0x0300, /* Combining Diacritical Marks			     */
       0x0370, /* Greek and Coptic				     */
       0x0400, /* Cyrillic					     */
       0x0500, /* Cyrillic Supplement				     */
       0x0530, /* Armenian					     */
-      0x0590, /* Hebrew						     */
-      0x0600, /* Arabic						     */
-      0x0700, /* Syriac						     */
+      0x0590, /* Hebrew					     */
+      0x0600, /* Arabic					     */
+      0x0700, /* Syriac					     */
       0x0750, /* Arabic Supplement				     */
-      0x0780, /* Thaana						     */
+      0x0780, /* Thaana					     */
       0x07C0, /* NKo						     */ /* 0x10 */
       0x0800, /* Samaritan					     */
       0x0840, /* Mandaic					     */
@@ -86,7 +86,7 @@ static long ucs_range[]={
       0x0A80, /* Gujarati					     */
       0x0B00, /* Oriya						     */
       0x0B80, /* Tamil						     */
-      0x0C00, /* Telugu						     */
+      0x0C00, /* Telugu					     */
       0x0C80, /* Kannada					     */
       0x0D00, /* Malayalam					     */
       0x0D80, /* Sinhala					     */
@@ -108,9 +108,9 @@ static long ucs_range[]={
       0x1760, /* Tagbanwa					     */
       0x1780, /* Khmer						     */
       0x1800, /* Mongolian					     */ /* 0x30 */
-      0x18B0, /* Unified Canadian Aboriginal Syllabics Extended	     */
+      0x18B0, /* Unified Canadian Aboriginal Syllabics Extended      */
       0x1900, /* Limbu						     */
-      0x1950, /* Tai Le						     */
+      0x1950, /* Tai Le					     */
       0x1980, /* New Tai Lue					     */
       0x19E0, /* Khmer Symbols					     */
       0x1A00, /* Buginese					     */
@@ -119,30 +119,31 @@ static long ucs_range[]={
       0x1B00, /* Balinese					     */
       0x1B80, /* Sundanese					     */
       0x1BC0, /* Batak						     */
-      0x1C00, /* Lepcha						     */
+      0x1C00, /* Lepcha					     */
       0x1C50, /* Ol Chiki					     */
       0x1C80, /* Cyrillic Extended-C				     */
-      0x1CC0, /* Sundanese Supplement				     */
-      0x1CD0, /* Vedic Extensions				     */ /* 0x40 */
+      0x1C90, /* Georgian Extended				     */
+      0x1CC0, /* Sundanese Supplement				     */ /* 0x40 */
+      0x1CD0, /* Vedic Extensions				     */
       0x1D00, /* Phonetic Extensions				     */
-      0x1D80, /* Phonetic Extensions Supplement			     */
-      0x1DC0, /* Combining Diacritical Marks Supplement		     */
+      0x1D80, /* Phonetic Extensions Supplement		     */
+      0x1DC0, /* Combining Diacritical Marks Supplement	     */
       0x1E00, /* Latin Extended Additional			     */
-      0x1F00, /* Greek Extended					     */
+      0x1F00, /* Greek Extended				     */
       0x2000, /* General Punctuation				     */
       0x2070, /* Superscripts and Subscripts			     */
       0x20A0, /* Currency Symbols				     */
       0x20D0, /* Combining Diacritical Marks for Symbols	     */
       0x2100, /* Letterlike Symbols				     */
       0x2150, /* Number Forms					     */
-      0x2190, /* Arrows						     */
-      0x2200, /* Mathematical Operators				     */
+      0x2190, /* Arrows					     */
+      0x2200, /* Mathematical Operators			     */
       0x2300, /* Miscellaneous Technical			     */
-      0x2400, /* Control Pictures				     */
-      0x2440, /* Optical Character Recognition			     */ /* 0x50 */
-      0x2460, /* Enclosed Alphanumerics				     */
+      0x2400, /* Control Pictures				     */ /* 0x50 */
+      0x2440, /* Optical Character Recognition			     */
+      0x2460, /* Enclosed Alphanumerics			     */
       0x2500, /* Box Drawing					     */
-      0x2580, /* Block Elements					     */
+      0x2580, /* Block Elements				     */
       0x25A0, /* Geometric Shapes				     */
       0x2600, /* Miscellaneous Symbols				     */
       0x2700, /* Dingbats					     */
@@ -154,8 +155,8 @@ static long ucs_range[]={
       0x2A00, /* Supplemental Mathematical Operators		     */
       0x2B00, /* Miscellaneous Symbols and Arrows		     */
       0x2C00, /* Glagolitic					     */
-      0x2C60, /* Latin Extended-C				     */
-      0x2C80, /* Coptic						     */ /* 0x60 */
+      0x2C60, /* Latin Extended-C				     */ /* 0x60 */
+      0x2C80, /* Coptic					     */
       0x2D00, /* Georgian Supplement				     */
       0x2D30, /* Tifinagh					     */
       0x2D80, /* Ethiopic Extended				     */
@@ -169,15 +170,15 @@ static long ucs_range[]={
       0x30A0, /* Katakana					     */
       0x3100, /* Bopomofo					     */
       0x3130, /* Hangul Compatibility Jamo			     */
-      0x3190, /* Kanbun						     */
-      0x31A0, /* Bopomofo Extended				     */
-      0x31C0, /* CJK Strokes					     */ /* 0x70 */
+      0x3190, /* Kanbun					     */
+      0x31A0, /* Bopomofo Extended				     */ /* 0x70 */
+      0x31C0, /* CJK Strokes					     */
       0x31F0, /* Katakana Phonetic Extensions			     */
       0x3200, /* Enclosed CJK Letters and Months		     */
       0x3300, /* CJK Compatibility				     */
       0x3400, /* CJK Unified Ideographs Extension A		     */
       0x4DC0, /* Yijing Hexagram Symbols			     */
-      0x4E00, /* CJK Unified Ideographs				     */
+      0x4E00, /* CJK Unified Ideographs			     */
       0xA000, /* Yi Syllables					     */
       0xA490, /* Yi Radicals					     */
       0xA4D0, /* Lisu						     */
@@ -186,14 +187,14 @@ static long ucs_range[]={
       0xA6A0, /* Bamum						     */
       0xA700, /* Modifier Tone Letters				     */
       0xA720, /* Latin Extended-D				     */
-      0xA800, /* Syloti Nagri					     */
-      0xA830, /* Common Indic Number Forms			     */ /* 0x80 */
+      0xA800, /* Syloti Nagri					     */ /* 0x80 */
+      0xA830, /* Common Indic Number Forms			     */
       0xA840, /* Phags-pa					     */
       0xA880, /* Saurashtra					     */
       0xA8E0, /* Devanagari Extended				     */
       0xA900, /* Kayah Li					     */
-      0xA930, /* Rejang						     */
-      0xA960, /* Hangul Jamo Extended-A				     */
+      0xA930, /* Rejang					     */
+      0xA960, /* Hangul Jamo Extended-A			     */
       0xA980, /* Javanese					     */
       0xA9E0, /* Myanmar Extended-B				     */
       0xAA00, /* Cham						     */
@@ -202,31 +203,31 @@ static long ucs_range[]={
       0xAAE0, /* Meetei Mayek Extensions			     */
       0xAB00, /* Ethiopic Extended-A				     */
       0xAB30, /* Latin Extended-E				     */
-      0xAB70, /* Cherokee Supplement				     */
-      0xABC0, /* Meetei Mayek					     */ /* 0x90 */
+      0xAB70, /* Cherokee Supplement				     */ /* 0x90 */
+      0xABC0, /* Meetei Mayek					     */
       0xAC00, /* Hangul Syllables				     */
-      0xD7B0, /* Hangul Jamo Extended-B				     */
+      0xD7B0, /* Hangul Jamo Extended-B			     */
       0xD800, /* High Surrogates				     */
       0xDB80, /* High Private Use Surrogates			     */
-      0xDC00, /* Low Surrogates					     */
+      0xDC00, /* Low Surrogates				     */
       0xE000, /* Private Use Area				     */
       0xF900, /* CJK Compatibility Ideographs			     */
       0xFB00, /* Alphabetic Presentation Forms			     */
       0xFB50, /* Arabic Presentation Forms-A			     */
       0xFE00, /* Variation Selectors				     */
-      0xFE10, /* Vertical Forms					     */
+      0xFE10, /* Vertical Forms				     */
       0xFE20, /* Combining Half Marks				     */
       0xFE30, /* CJK Compatibility Forms			     */
       0xFE50, /* Small Form Variants				     */
-      0xFE70, /* Arabic Presentation Forms-B			     */
-      0xFF00, /* Halfwidth and Fullwidth Forms			     */ /* 0xa0 */
-      0xFFF0, /* Specials                               	     */
+      0xFE70, /* Arabic Presentation Forms-B			     */ /* 0xa0 */
+      0xFF00, /* Halfwidth and Fullwidth Forms			     */
+      0xFFF0, /* Specials					     */
       0x10000, /* Linear B Syllabary				     */
       0x10080, /* Linear B Ideograms				     */
       0x10100, /* Aegean Numbers				     */
-      0x10140, /* Ancient Greek Numbers				     */
+      0x10140, /* Ancient Greek Numbers			     */
       0x10190, /* Ancient Symbols				     */
-      0x101D0, /* Phaistos Disc					     */
+      0x101D0, /* Phaistos Disc				     */
       0x10280, /* Lycian					     */
       0x102A0, /* Carian					     */
       0x102E0, /* Coptic Epact Numbers				     */
@@ -234,11 +235,11 @@ static long ucs_range[]={
       0x10330, /* Gothic					     */
       0x10350, /* Old Permic					     */
       0x10380, /* Ugaritic					     */
-      0x103A0, /* Old Persian					     */
-      0x10400, /* Deseret					     */ /* 0xb0 */
+      0x103A0, /* Old Persian					     */ /* 0xb0 */
+      0x10400, /* Deseret					     */
       0x10450, /* Shavian					     */
       0x10480, /* Osmanya					     */
-      0x104B0, /* Osage						     */
+      0x104B0, /* Osage					     */
       0x10500, /* Elbasan					     */
       0x10530, /* Caucasian Albanian				     */
       0x10600, /* Linear A					     */
@@ -250,24 +251,28 @@ static long ucs_range[]={
       0x10900, /* Phoenician					     */
       0x10920, /* Lydian					     */
       0x10980, /* Meroitic Hieroglyphs				     */
-      0x109A0, /* Meroitic Cursive				     */
-      0x10A00, /* Kharoshthi					     */ /* 0xc0 */
+      0x109A0, /* Meroitic Cursive				     */ /* 0xc0 */
+      0x10A00, /* Kharoshthi					     */
       0x10A60, /* Old South Arabian				     */
       0x10A80, /* Old North Arabian				     */
       0x10AC0, /* Manichaean					     */
       0x10B00, /* Avestan					     */
       0x10B40, /* Inscriptional Parthian			     */
-      0x10B60, /* Inscriptional Pahlavi				     */
+      0x10B60, /* Inscriptional Pahlavi			     */
       0x10B80, /* Psalter Pahlavi				     */
       0x10C00, /* Old Turkic					     */
-      0x10C80, /* Old Hungarian					     */
+      0x10C80, /* Old Hungarian				     */
+      0x10D00, /* Hanifi Rohingya				     */
       0x10E60, /* Rumi Numeral Symbols				     */
-      0x11000, /* Brahmi					     */
+      0x10F00, /* Old Sogdian					     */
+      0x10F30, /* Sogdian					     */
+      0x10FE0, /* Elymaic					     */
+      0x11000, /* Brahmi					     */ /* 0xd0 */
       0x11080, /* Kaithi					     */
       0x110D0, /* Sora Sompeng					     */
       0x11100, /* Chakma					     */
       0x11150, /* Mahajani					     */
-      0x11180, /* Sharada					     */ /* 0xd0 */
+      0x11180, /* Sharada					     */
       0x111E0, /* Sinhala Archaic Numbers			     */
       0x11200, /* Khojki					     */
       0x11280, /* Multani					     */
@@ -278,60 +283,75 @@ static long ucs_range[]={
       0x11580, /* Siddham					     */
       0x11600, /* Modi						     */
       0x11660, /* Mongolian Supplement				     */
-      0x11680, /* Takri						     */
+      0x11680, /* Takri					     */ /* 0xe0 */
       0x11700, /* Ahom						     */
+      0x11800, /* Dogra					     */
       0x118A0, /* Warang Citi					     */
+      0x119A0, /* Nandinagari					     */
       0x11A00, /* Zanabazar Square				     */
       0x11A50, /* Soyombo					     */
-      0x11AC0, /* Pau Cin Hau					     */ /* 0xe0 */
+      0x11AC0, /* Pau Cin Hau					     */
       0x11C00, /* Bhaiksuki					     */
       0x11C70, /* Marchen					     */
-      0x11D00, /* Masaram Gondi					     */
+      0x11D00, /* Masaram Gondi				     */
+      0x11D60, /* Gunjala Gondi				     */
+      0x11EE0, /* Makasar					     */
+      0x11FC0, /* Tamil Supplement				     */
       0x12000, /* Cuneiform					     */
       0x12400, /* Cuneiform Numbers and Punctuation		     */
-      0x12480, /* Early Dynastic Cuneiform			     */
+      0x12480, /* Early Dynastic Cuneiform			     */ /* 0xf0 */
       0x13000, /* Egyptian Hieroglyphs				     */
-      0x14400, /* Anatolian Hieroglyphs				     */
+      0x13430, /* Egyptian Hieroglyph Format Controls		     */
+      0x14400, /* Anatolian Hieroglyphs			     */
       0x16800, /* Bamum Supplement				     */
       0x16A40, /* Mro						     */
       0x16AD0, /* Bassa Vah					     */
       0x16B00, /* Pahawh Hmong					     */
+      0x16E40, /* Medefaidrin					     */
       0x16F00, /* Miao						     */
       0x16FE0, /* Ideographic Symbols and Punctuation		     */
       0x17000, /* Tangut					     */
-      0x18800, /* Tangut Components				     */ /* 0xf0 */
+      0x18800, /* Tangut Components				     */
       0x1B000, /* Kana Supplement				     */
       0x1B100, /* Kana Extended-A				     */
-      0x1B170, /* Nushu						     */
+      0x1B130, /* Small Kana Extension				     */
+      0x1B170, /* Nushu					     */ /* 0x100 */
       0x1BC00, /* Duployan					     */
       0x1BCA0, /* Shorthand Format Controls			     */
       0x1D000, /* Byzantine Musical Symbols			     */
       0x1D100, /* Musical Symbols				     */
       0x1D200, /* Ancient Greek Musical Notation		     */
-      0x1D300, /* Tai Xuan Jing Symbols				     */
-      0x1D360, /* Counting Rod Numerals				     */
+      0x1D2E0, /* Mayan Numerals				     */
+      0x1D300, /* Tai Xuan Jing Symbols			     */
+      0x1D360, /* Counting Rod Numerals			     */
       0x1D400, /* Mathematical Alphanumeric Symbols		     */
       0x1D800, /* Sutton SignWriting				     */
-      0x1E000, /* Glagolitic Supplement				     */
-      0x1E800, /* Mende Kikakui					     */
-      0x1E900, /* Adlam						     */
-      0x1EE00, /* Arabic Mathematical Alphabetic Symbols	     */ /* 0x100 */
-      0x1F000, /* Mahjong Tiles					     */
+      0x1E000, /* Glagolitic Supplement			     */
+      0x1E100, /* Nyiakeng Puachue Hmong			     */
+      0x1E2C0, /* Wancho					     */
+      0x1E800, /* Mende Kikakui				     */
+      0x1E900, /* Adlam					     */
+      0x1EC70, /* Indic Siyaq Numbers				     */ /* 0x110 */
+      0x1ED00, /* Ottoman Siyaq Numbers			     */
+      0x1EE00, /* Arabic Mathematical Alphabetic Symbols	     */
+      0x1F000, /* Mahjong Tiles				     */
       0x1F030, /* Domino Tiles					     */
-      0x1F0A0, /* Playing Cards					     */
+      0x1F0A0, /* Playing Cards				     */
       0x1F100, /* Enclosed Alphanumeric Supplement		     */
       0x1F200, /* Enclosed Ideographic Supplement		     */
-      0x1F300, /* Miscellaneous Symbols and Pictographs		     */
+      0x1F300, /* Miscellaneous Symbols and Pictographs	     */
       0x1F600, /* Emoticons					     */
       0x1F650, /* Ornamental Dingbats				     */
       0x1F680, /* Transport and Map Symbols			     */
       0x1F700, /* Alchemical Symbols				     */
       0x1F780, /* Geometric Shapes Extended			     */
-      0x1F800, /* Supplemental Arrows-C				     */
+      0x1F800, /* Supplemental Arrows-C			     */
       0x1F900, /* Supplemental Symbols and Pictographs		     */
+      0x1FA00, /* Chess Symbols				     */ /* 0x120 */
+      0x1FA70, /* Symbols and Pictographs Extended-A		     */
       0x20000, /* CJK Unified Ideographs Extension B		     */
       0x2A700, /* CJK Unified Ideographs Extension C		     */
-      0x2B740, /* CJK Unified Ideographs Extension D		     */ /* 0x110 */
+      0x2B740, /* CJK Unified Ideographs Extension D		     */
       0x2B820, /* CJK Unified Ideographs Extension E		     */
       0x2CEB0, /* CJK Unified Ideographs Extension F		     */
       0x2F800, /* CJK Compatibility Ideographs Supplement	     */
@@ -343,11 +363,11 @@ static long ucs_range[]={
       0x80000, /* reserved					     */
       0x90000, /* reserved					     */
       0xA0000, /* reserved					     */
-      0xB0000, /* reserved					     */
+      0xB0000, /* reserved					     */ /* 0x130 */
       0xC0000, /* reserved					     */
       0xD0000, /* reserved					     */
       0xE0000, /* Tags						     */
-      0xE0100, /* Variation Selectors Supplement		     */ /* 0x120 */
+      0xE0100, /* Variation Selectors Supplement		     */
       0xF0000, /* Supplementary Private Use Area-A		     */
       0x100000, /* Supplementary Private Use Area-B		     */
   /* Value over 0x10FFFF is illegal under Unicode,
@@ -361,11 +381,11 @@ static long ucs_range[]={
       0x170000, /* Reserved					     */
       0x180000, /* Reserved					     */
       0x190000, /* Reserved					     */
-      0x1A0000, /* Reserved					     */
+      0x1A0000, /* Reserved					     */ /* 0x140 */
       0x1B0000, /* Reserved					     */
       0x1C0000, /* Reserved					     */
       0x1D0000, /* Reserved					     */
-      0x1E0000, /* Reserved					     */ /* 0x130 */
+      0x1E0000, /* Reserved					     */
       0x1F0000, /* Reserved					     */
       0x200000, /* Reserved					     */
       0x210000, /* Reserved					     */
@@ -424,7 +444,7 @@ integer kcatcodekey(integer c)
                || (LATIN_SMALL_LETTER_O_WITH_STROKE  <=c && c<=LATIN_SMALL_LETTER_Y_WITH_DIAERESIS  ) )
             return 0x1FD;
         }
-        if (block==0xa0) {
+        if (block==0xa1) {
             /* Fullwidth ASCII variants  except for U+FF01..FF0F, U+FF1A..FF20, U+FF3B..FF40, U+FF5B..FF5E */
             if (  (FULLWIDTH_DIGIT_0  <=c && c<=FULLWIDTH_DIGIT_9  )
                || (FULLWIDTH_CAPITAL_A<=c && c<=FULLWIDTH_CAPITAL_Z)
@@ -465,8 +485,6 @@ void init_default_kanji (const_string file_str, const_string internal_str)
 {
     char *p;
 
-    enable_UPTEX (true); /* enable */
-
     init_kanji (file_str, internal_str);
 
     p = getenv ("PTEX_KANJI_ENC");
@@ -484,3 +502,33 @@ void init_default_kanji (const_string file_str, const_string internal_str)
     }
 #endif
 }
+
+void init_default_kanji_select(void)
+{
+    char *base;
+
+    base = kpse_program_basename (argv[0]);
+
+    if (FILESTRNCASEEQ(base, "p", 1) || FILESTRNCASEEQ(base, "ep", 2)) {
+
+        enable_UPTEX (false); /* disable */
+#if defined(WIN32)
+/* pBibTeX is EUC only */
+        if (FILESTRNCASEEQ(base, "pbibtex", 7)) {
+            init_default_kanji(NULL, "euc");
+        } else {
+/* for pTeX, e-pTeX, pDVItype, pPLtoTF, and pTFtoPL */
+            init_default_kanji(NULL, "sjis");
+        }
+#else
+        init_default_kanji(NULL, "euc");
+#endif
+
+    } else {
+
+/* for upTeX, e-upTeX, upBibTeX, upDVItype, upPLtoTF, and upTFtoPL */
+        enable_UPTEX (true);  /* enable */
+        init_default_kanji ("utf8", "uptex");
+
+    }
+}
